//
//  listViewController.m
//  list
//
//  Created by Lucas on 10/9/13.
//
//

#import "listViewController.h"
#import <objc/runtime.h>

@interface listViewController ()

@property (nonatomic, strong) NSMutableArray * tableData;

@end

static char indexPathKey;

@implementation listViewController

- (void)viewDidLoad
{
    [super viewDidLoad];
    self.navigationItem.leftBarButtonItem = self.editButtonItem;
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return [self.tableData count];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *CellIdentifier = @"Cell";
    
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    
    // Configure the cell...
    if (cell == nil) {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier];
    }
    
    // Display name in cell's UITextField
    NSString *name = [self.tableData objectAtIndex:indexPath.row];
    
    UITextField *nameField = (UITextField *)[cell viewWithTag:101];
    nameField.text = name;
    
    // Associate new item cell's text field with its tableData index
    objc_setAssociatedObject(nameField, &indexPathKey, indexPath, OBJC_ASSOCIATION_RETAIN);
    
    return cell;
}

- (IBAction)textFieldFinished:(id)sender
{
    UITextField *nameField = (UITextField *)sender;
    NSString *fieldText = nameField.text;
    
    // save to tableData on finished editing UITextField
    NSIndexPath *indexPath = objc_getAssociatedObject(nameField, &indexPathKey);
    self.tableData[indexPath.row] = fieldText;
    [self saveList];
}

- (void)addItem
{
    // end editing if currently editing something
    [self.view endEditing:YES];
    
    int newRowIndex = 0;
    [self.tableData insertObject:@"" atIndex:newRowIndex];
    
    NSIndexPath *indexPath = [NSIndexPath indexPathForRow:newRowIndex inSection:0];
    NSArray *indexPaths = [NSArray arrayWithObject:indexPath];
    [self.tableView insertRowsAtIndexPaths:indexPaths withRowAnimation:UITableViewRowAnimationFade];
    
    [self.tableView reloadData];
    
    // make new item cell's text field first responder
    NSIndexPath *firstIndexPath = [NSIndexPath indexPathForRow:0 inSection:0];
    UITableViewCell *cell = [self.tableView cellForRowAtIndexPath:firstIndexPath];
    UITextField *nameField = (UITextField *)[cell viewWithTag:101];
    [nameField becomeFirstResponder];
}

- (void)loadList
{
    NSString *path = [self dataFilePath];
    if ([[NSFileManager defaultManager] fileExistsAtPath:path])
    {
        NSData *data = [[NSData alloc] initWithContentsOfFile:path];
        NSKeyedUnarchiver *unarchiver = [[NSKeyedUnarchiver alloc] initForReadingWithData:data];
        self.tableData = [unarchiver decodeObjectForKey:@"TableData"];
        [unarchiver finishDecoding];
    }
    else
    {
        self.tableData = [[NSMutableArray alloc] initWithCapacity:20];
    }
}

- (id)initWithCoder:(NSCoder *)aDecoder
{
    if ((self = [super initWithCoder:aDecoder])) {
        [self loadList];
    }
    return self;
}

- (NSString *)documentsDirectory
{
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *documentsDirectory = [paths objectAtIndex:0];
    return documentsDirectory;
}

- (NSString *)dataFilePath
{
    return [[self documentsDirectory] stringByAppendingPathComponent:@"Checklists.plist"];
}

- (void)saveList
{
    NSMutableData *data = [[NSMutableData alloc] init];
    NSKeyedArchiver *archiver = [[NSKeyedArchiver alloc] initForWritingWithMutableData:data];
    [archiver encodeObject:self.tableData forKey:@"TableData"];
    [archiver finishEncoding];
    [data writeToFile:[self dataFilePath] atomically:YES];
}

- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath
{
    [self.tableData removeObjectAtIndex:indexPath.row];
    
    [self saveList];
    
    NSArray *indexPaths = [NSArray arrayWithObject:indexPath];
    [tableView deleteRowsAtIndexPaths:indexPaths withRowAnimation:UITableViewRowAnimationFade];
}

- (IBAction)add
{
    [self addItem];
}

- (BOOL)tableView:(UITableView *)tableview canMoveRowAtIndexPath:(NSIndexPath *)indexPath
{
    return TRUE;
}

- (void)tableView:(UITableView *)tableView moveRowAtIndexPath:(NSIndexPath *)fromIndexPath toIndexPath:(NSIndexPath *)toIndexPath
{
    id buffer = [self.tableData objectAtIndex:fromIndexPath.row];
    [self.tableData removeObjectAtIndex:fromIndexPath.row];
    [self.tableData insertObject:buffer atIndex: toIndexPath.row];
}

- (void)scrollViewWillBeginDragging:(UIScrollView *)scrollView
{
    // dismiss keyboard on scroll
    [self.view endEditing:YES];
}

@end
